import numpy as np


def read_file(file_name, out_list=True):
    
    open_file = open(file_name, 'r')
    if out_list:
        text = open_file.readlines()
    else:
        text = open_file.read()
    open_file.close()
    return text


def write_file(file_name, text, key='w'):

    new_file = open(file_name, key)
    if text is list:
        new_file.writelines(text)
    else:
        new_file.write(text)
    new_file.close()
    return None


def read_pdb(file_name):

    lines = read_file(file_name + ".pdb")
    symbols = []
    coordinates = []
    edges = []
    for l in lines:
        if "HETATM" in l or "ATOM" in l:
            s_line = l.split()
            symbols += [s_line[-1]]
            coordinates += [[float(c) for c in s_line[-6: -3]]]
        if "CONECT" in l:
            s_line = l.split()
            for ind in s_line[2:]:
                if int(s_line[1]) < int(ind):
                    edges += [(int(s_line[1])-1, int(ind)-1)]
                    edges += [(int(ind)-1, int(s_line[1])-1)]
    edges = list(set(edges))
    edges = [[v1, v2] for v1, v2 in edges if v1 < v2]
    edges.sort()
    return np.array(symbols), np.array(coordinates), np.array(edges)


def write_pdb(file_name, symbols, coordinates, edges):
    
    text = "REMARK    1 File created by GraphSplitter\n"
    for i in range(len(symbols)):
        text += ("HETATM"
                 + "%5d" % (i + 1)
                 + "%4s" % symbols[i]
                 + "%23.3f" % coordinates[i][0]
                 + "%8.3f" % coordinates[i][1]
                 + "%8.3f" % coordinates[i][2]
                 + "%6.2f" % 1.00
                 + "%6.2f" % 1.00
                 + "%12s" % symbols[i] + '\n')
    text += "END\n"
    adjacency_list = [[] for i in range(len(symbols))]
    for v_1, v_2 in edges:
        adjacency_list[v_1] += [v_2]
        adjacency_list[v_2] += [v_1]
    for i, neighbors in enumerate(adjacency_list):
        connect = "CONECT" + "%5d" % (i+1)
        for n in neighbors:
            connect += "%5d" % (n + 1)
        connect += '\n' 
        text += connect
    write_file(file_name + ".pdb", text, 'w')
    return None
